/*
 * File:   newmainXC16.c
 * Author: M.
 *
 * Created on 10 April 2017, 20:40
 * 
 * example:
 * PIC UART1 is used for communication with BBB
 * At transmit 1Byte with value 0, 1 or 2 -> this lights
 * respectively LED 1L, 2L or 3L; other value turns off all LEDs
 * returns echo
 * LED 2H blinks with period 2s simply for indication.
 */

#include "newmainXC16.h"

// FOR DELAYS!!!
#define FCY 62500000ULL // 1/FCY -> 1/62,5MHz = 16ns
#include <libpic30.h>
#define delay_us(x) __delay_us(x)
#define delay_ms(x) __delay_ms(x)

#include <ports.h>
#include <pps.h>


// FUNCTIONs
// ---------


// FOR SYSTEM
// ==========

void SetClock(void) {
    // Quartz = 10MHz
    // PLLPRE = 2 (default) -> 5MHz
    // PLL = 56 -> 5*56 = 280MHz
    // PLLPOST = 2 (NOT default!) -> 140MHz, divided by 2 in CPU -> result max 70MHz
    // this is minimal command time = 1/70MHz = 14.2857ns
    // BUT:
    // if use PLL = 50 (default) =>
    // 5MHz -> 250MHz -> 125MHz -> 62,5MHz in CPU. i. e. 16ns step
    PLLFBD = 48; // M=50 // 0->2; ... 48->50 (default) ... 255->(err. in datasheet?!)
    _PLLPOST = 0; // N2=2 // 0->2; 1->4 (default); 3->8
    _PLLPRE = 0; // N1=2 // 0->2 (default) do 31->33

    __builtin_write_OSCCONH(3); // Initiate Clock Switch to Primary Oscillator with PLL (NOSC=0b011)
    __builtin_write_OSCCONL(OSCCON | 1); // Switch request
    // Wait for Clock switch to occur
    while (OSCCONbits.COSC != 3);
    // Wait for PLL to lock
    while (OSCCONbits.LOCK != 1);
}

void IOinit(void) {
    // in documentation - only LATA is 0 after reset
    LATA = LATB = LATC = 0;

    // OFF Analog function (at ADCinit will be set the used)
    ANSELA = ANSELB = ANSELC = 0;
    
    PPSInput(IN_FN_PPS_U1RX, IN_PIN_PPS_RPI34); // Rx UART1
    PPSOutput(OUT_FN_PPS_U1TX, OUT_PIN_PPS_RP35); // Tx UART1 
    
    // LOCK remapping
    PPSLock;
    
    // example outs
    _TRISB10 = 0;
    _TRISB11 = 0;
    _TRISB12 = 0;
    _TRISB13 = 0;
    _TRISB14 = 0;
    _TRISB15 = 0;
}

void UARTinit(void) {
    // BR = 115,2kbps
    // check: 62,5MHz/(16*(U1BRG+1)) = 62500k/(16*34) = 114,89k
    // error: <0,3%
    U1BRG = 33;
    _UTXISEL0 = 1; // Tx ISR if Transmit is fully completed
    _URXISEL = 1; // Rx ISR if Received 4B (full input buffer)
    _UARTEN = 1; // Switch on
    _UTXEN = 1; // Enable Tx
}


int main(void) {
    unsigned char tmp;
    SetClock();
    IOinit();
    UARTinit();

    // Main loop
    while (1) {
        delay_ms(1000);
        Out2H = !Out2H;
        
        // UART Communication
        if (U1STAbits.OERR || _U1RXIF){
            tmp = U1RXREG;
            U1STAbits.OERR = 0; _U1RXIF = 0;
            switch (tmp){
                case 0: Out1L = 1; break;
                case 1: Out2L = 1; break;
                case 2: Out3L = 1; break;
                default: Out1L=0; Out2L=0; Out3L=0; break;
            }        
            U1TXREG = tmp;
        }
    }

    return 0;
}
